;
         TITLE "Minuterie Programmable"

;        Date de cration    : 18/09/95
;        Date de Mise  jour : 28/01/96
;        Auteur              ; Michel OUSSET
;        Cadencement         : 12Mhz

;

         LIST    C=132, T=OFF, P=16C54, R=DEC, N=100

PIC54    EQU     1FFH        ;Dfinition du Reset Vector

RTCC     EQU     01          ;Adresse du Real Time Clock/Counter register (f1)
PC       EQU     02          ;Program Counter                             (f2)   
STAT     EQU     03          ;Status Word Register                        (f3)
FSR      EQU     04          ;File Select Register                        (f4)   

PORTA    EQU     05          ;Adresse du Port A                           (f5)
PORTB    EQU     06          ;Adresse du Port B                           (f6)

CARRY    EQU     00          ;Position du bit Carry dans f3
ZBIT     EQU     02          ;Position du bit Zero dans f3

;
;        Dfinition des connexions d'Entre-Sortie
;

;        RA0 == Pds faible Minutes   (actif tat 0)  Sortie
;        RA1 == Pds fort   Minutes   (actif tat 0)  Sortie
;        RA2 == Pds fort   Secondes  (actif tat 0)  Sortie
;        RA3 == Pds faible Secondes  (actif tat 0)  Sortie
                                                    
;        RB0 == Segment g | Bouton 0 (actif tat 1)  Sortie|Sortie
;        RB1 == Segment f | Bouton 1 (actif tat 1)  Sortie|Sortie
;        RB2 == Segment b | Bouton 2 (actif tat 1)  Sortie|Sortie
;        RB3 == Segment a | Bouton 3 (actif tat 1)  Sortie|Sortie
;        RB4 == Segment e | Bouton 4 (actif tat 1)  Sortie|Sortie
;        RB5 == Segment d            (actif tat 1)  Sortie
;        RB6 == Segment c | Commun   (actif tat 1)  Sortie|Entre

;        RB7 == Sortie relais        (actif tat 1)  Sortie

;        Bouton 0 (RB0) : Rglage Minutes
;        Bouton 1 (RB1) : Rglage Secondes
;        Bouton 2 (RB2) : Slection dcrmentation
;        Bouton 3 (RB3) : Dmarrage / Arrt dcomptage (Start/Stop)
;        Bouton 4 (RB4) : Marche / Arrt immdiat (M/A)

;
;        Dclaration des variables 
;
;        De 08  31 (1F) (25 octets)
;

MU       EQU     08          ;Pds faible minutes   Affichage           (1)
MD       EQU     09          ;Pds fort   minutes   Affichage           (2)
SU       EQU     10          ;Pds faible secondes  Affichage           (3)
SD       EQU     11          ;Pds fort   secondes  Affichage           (4)
MUC      EQU     12          ;Pds faible minutes   Consigne            (5)
MDC      EQU     13          ;Pds fort   minutes   Consigne            (6)
SUC      EQU     14          ;Pds faible secondes  Consigne            (7)
SDC      EQU     15          ;Pds fort   secondes  Consigne            (8)
DIGIT    EQU     16          ;Digit  afficher                         (9)
TIMER    EQU     17          ;Compteur de temps                        (10)
TMIN     EQU     18          ;Dcodage Touche Minute  (RB0)            (11)
TSEC     EQU     19          ;Dcodage Touche Seconde (RB1)            (12)
TSTART   EQU     20          ;Dcodage Touche Start   (RB2)            (13)
TSTOP    EQU     21          ;Dcodage Touche Stop    (RB3)            (14)
TDOWN    EQU     22          ;Dcodage Touche Down    (RB4)            (15)
REPET0   EQU     23          ;Tempo pour la rptition touche Minute   (16)
REPET1   EQU     24          ;Tempo pour la rptition touche Seconde  (17)
ETAT     EQU     25          ;Etat de fonctionnement du Timer          (18)

;
;        Dclaration des paramtres constants
;

SECONDE  EQU     183         ;Nbr d'impulsions dans timer pour 1 seconde
TMPREPET EQU     40          ;Incrment chaque 0.22 seconde

;
;        Dbut du programme 
;

MAIN     CALL    INIT        ;Initialisation du programme

LOOP     CLRWDT              ;Reset du Watchdog

; Teste digit  rafraichir

         BTFSC   RTCC,6      ;Skip si RTCC diffrent de 64
                             ;RTCC = compteur incrment  chaque impulsion

         CALL    PRINC

         GOTO    LOOP

;
;        Initialisation du programme 
;

INIT     CLRWDT              ;Reset du watchdog

; Initialisation du PORT A en sortie

         MOVLW   b'11110000' ;'0' = sortie, '1' = entre
         TRIS    PORTA

         MOVLW   b'00001111' ;Initialisation des sorties A  1
         MOVWF   PORTA

; Initialisation du PORT B en sortie

         MOVLW   b'00000000' ;'0' = sortie, '1' = entre
         TRIS    PORTB

         MOVLW   b'00000000' ;Initialisation des sorties B  0
         MOVWF   PORTB

; Initialisation de l'Option Register
;
;        PS2  PS1  PS0  RTCC   WDT         B3 : PSA (Prescaler Assignement bit)
;         0    0    0   1:2    1:1              0 == RTCC ; 1 == WDT
;         0    0    1   1:4    1:2              
;         0    1    0   1:8    1:4         B4 : RTE (RTCC Signal Edge)
;         0    1    1   1:16   1:8              0 == Low-to-High ; 1 == High-to-Low
;         1    0    0   1:32   1:16             
;         1    0    1   1:64   1:32        B5 : RTS (RTCC Signal Source)
;         1    1    0   1:128  1:64             0 == Internal ; 1 == RTCC Pin
;         1    1    1   1:256  1:128            

         MOVLW   b'000111'   ;WDT  18ms, fos/4 / 256 (RTCC incrment) = 85s
         OPTION

; Initialisation des variables

         CLRF    DIGIT       ;Digit = 0
         CLRF    MD          ;Unit des minutes = 0
         CLRF    MU          ;Dizaine des minutes = 0
         CLRF    SU          ;Unit des secondes = 0
         CLRF    SD          ;Dizaine des secondes = 0
         CLRF    MUC         ;Consigne unit minute = 0
         CLRF    MDC         ;Consigne dizaine minute = 0
         CLRF    SUC         ;Consigne unit seconde = 0
         CLRF    SDC         ;Consigne dizaine seconde = 0
         MOVLW   SECONDE
         MOVWF   TIMER       ;Compteur de temps = SECONDE impulsions pour 1 seconde
         CLRF    ETAT
         
         RETLW   00          ;Fin de la procdure d'initialisation               

;
;        Procdure de gestion de l'affichage
;
;        Affichage de [MD MU : SD SU]
;        Appel tous les 5,46ms (12E6/4/256/64 = 183,1hz)
;

GESAFF

; Prparation de l'affichage

         MOVF    DIGIT,W
         ADDWF   PC

         GOTO    DIGIT0      ;Affichage DIGIT0 : Poids fort   Minutes  (MD)
         GOTO    DIGIT1      ;Affichage DIGIT1 : Poids faible Minutes  (MU)
         GOTO    DIGIT2      ;Affichage DIGIT2 : Poids fort   Secondes (SD)
         GOTO    DIGIT3      ;Affichage DIGIT3 : Poids faible Secondes (SU)

DIGIT0   MOVLW   1           ;Future valeur de DIGIT
         MOVWF   DIGIT
         RLF     MD,W        ;Tranfert MD*2 dans W (Digit 0)
         BSF     PORTA,3     ;DIGIT3 = off
         BCF     PORTA,1     ;DIGIT0 = on
         GOTO    AFFDIG

DIGIT1   MOVLW   2           ;Future valeur de DIGIT
         MOVWF   DIGIT
         RLF     MU,W        ;Tranfert MU*2 dans W (Digit 1)
         BSF     PORTA,1     ;DIGIT0 = off
         BCF     PORTA,0     ;DIGIT1 = on
         GOTO    AFFDIG

DIGIT2   MOVLW   3           ;Future valeur de DIGIT
         MOVWF   DIGIT
         RLF     SD,W        ;Tranfert SD*2 dans W (Digit 2)
         BSF     PORTA,0     ;DIGIT1 = off
         BCF     PORTA,2     ;DIGIT2 = on
         GOTO    AFFDIG

DIGIT3   MOVLW   0           ;Future valeur de DIGIT
         MOVWF   DIGIT
         RLF     SU,W        ;Tranfert SU*2 dans W (Digit 3)
         BSF     PORTA,2     ;DIGIT2 = off
         BCF     PORTA,3     ;DIGIT3 = on
         GOTO    AFFDIG
        
AFFDIG   ADDWF   PC          ;Accs au dcodage des segments (cdeabfg)

;Ordre des segments CDEABFG

; '0'        
         MOVLW   b'01111110'
         GOTO    AFFOUT

; '1'
         MOVLW   b'01000100'
         GOTO    AFFOUT

; '2'
         MOVLW   b'00111101'
         GOTO    AFFOUT

; '3'
         MOVLW   b'01101101'
         GOTO    AFFOUT

; '4'
         MOVLW   b'01000111'
         GOTO    AFFOUT

; '5'
         MOVLW   b'01101011'
         GOTO    AFFOUT

; '6'
         MOVLW   b'01111011'
         GOTO    AFFOUT

; '7'
         MOVLW   b'01001100'
         GOTO    AFFOUT

; '8'
         MOVLW   b'01111111'
         GOTO    AFFOUT

; '9'
         MOVLW   b'01101111'
         GOTO    AFFOUT

AFFOUT   BTFSC   PORTB,7     ;Teste l'tat de la sortie RB7
         IORLW   b'10000000' ;Copie le contenu de RB7 dans W
         MOVWF   PORTB       ;Commande des segments
         GOTO    AFFEND

AFFEND   RETLW   00

;
;        Procdure principale
;
;        Appel tous les 5,46ms (12E6/4/256/64 = 183,1hz)
;

PRINC    CLRF    RTCC        ;Base de temps de rafraichissement  0

         DECFSZ  TIMER       ;Dcrmente et skip si timer est arriv  0
         GOTO    PRINC1      ;Continue l'affichage

         MOVLW   SECONDE     ;Rinitialise TIMER
         MOVWF   TIMER

         BTFSS   ETAT,0      ;Skip si temporisation en cours
         GOTO    PRINC1

; Teste si [MD MU : SD SU] = 00:00

         CLRW                ;Prparation du test 
         SUBWF   MD,0        ; Excute MD - W dans W
         BTFSS   STAT,ZBIT   ;Skip si MD = 0
         GOTO    PRINC2      ;Dcomptage
         SUBWF   MU,0        ; Excute MU - W dans W
         BTFSS   STAT,ZBIT   ;Skip si MU = 0
         GOTO    PRINC2      ;Dcomptage
         SUBWF   SD,0        ; Excute SD - W dans W
         BTFSS   STAT,ZBIT   ;Skip si SD = 0
         GOTO    PRINC2      ;Dcomptage
         SUBWF   SU,0        ; Excute SU - W dans W
         BTFSS   STAT,ZBIT   ;Skip si SU = 0
         GOTO    PRINC2      ;Dcomptage
         BCF     ETAT,0      ;Mmorise fin de tempo
         MOVFW   PORTB       ;Bascule RB7, Lecture du PORTB
         XORLW   b'10000000' ;Bascule Bit7(PORTB)
         MOVWF   PORTB       ;Ecriture du PORTB
         GOTO    PRINC1

PRINC2   CALL    DECAFF      ;Dcrmente MD MU : SD SU

PRINC1   CALL    KEY         ;Lecture du clavier
         CALL    GESCLAV     ;Gestion des ordres clavier
         CALL    GESAFF

         RETLW   00

;
;        Lecture du clavier

;        Variables modifies : TMIN, TSEC, TSTART, TSTOP, TDOWN

;        Structure de ces variables : 
;          Bit(7)    = 1 Touche appuye
;          Bit(6..0) = Compteur anti-rebond
                               
;        Appel tous les 5,46ms (12E6/4/256/64 = 183,1hz)
;

KEY      MOVLW   b'00001111' ;Eteint tous les afficheurs
         MOVWF   PORTA

                             ; Lecture touche minute (RB0)
         
         MOVLW   b'01111110' ;Met RB[6,5,4,3,2,1] en entre, RB0 en sortie
         TRIS    PORTB

         BSF     PORTB,0     ;Sonde la touche avec RB0 = 1

         BTFSS   PORTB,6     ;Skip si RB6 = 1 (Touche appuye)
         GOTO    KEY00       ;Saut car touche non appuye
         BTFSS   TMIN,7      ;Skip si touche dj appuye
         INCF    TMIN        ;Incrmente compteur
         GOTO    KEY01       ;Saut touche suivante
KEY00    MOVLW   128-4       ;128 == (Bit7 = 1)
         MOVWF   TMIN        ;Le Bit7 passera  1 aprs 4 appels OK
    
KEY01                        ; Lecture touche seconde (RB1)
         
         MOVLW   b'01111101' ;Met RB[6,5,4,3,2,0] en entre, RB1 en sortie
         TRIS    PORTB

         BSF     PORTB,1     ;Sonde la touche avec RB1 = 1

         BTFSS   PORTB,6     ;Skip si RB6 = 1 (Touche appuye)
         GOTO    KEY02       ;Saut car touche non appuye
         BTFSS   TSEC,7      ;Skip si touche dj appuye
         INCF    TSEC        ;Incrmente compteur
         GOTO    KEY03       ;Saut touche suivante
KEY02    MOVLW   128-4       ;128 == (Bit7 = 1)
         MOVWF   TSEC        ;Le Bit7 passera  1 aprs 4 appels OK

KEY03                        ; Lecture touche UP/DOWN (RB2)

         MOVLW   b'01111011' ;Met RB[6,5,4,3,1,0] en entre, RB2 en sortie
         TRIS    PORTB

         BSF     PORTB,2     ;Sonde la touche avec RB2 = 1

         BTFSS   PORTB,6     ;Skip si RB6 = 1 (Touche appuye)
         GOTO    KEY04       ;Saut car touche non appuye
         BTFSS   TDOWN,7     ;Skip si touche dj appuye
         INCF    TDOWN       ;Incrmente compteur
         GOTO    KEY05       ;Saut touche suivante
KEY04    MOVLW   128-4       ;128 == (Bit7 = 1)
         MOVWF   TDOWN       ;Le Bit7 passera  1 aprs 4 appels OK

KEY05                        ; Lecture touche Start/Stop  (RB3)
    
         MOVLW   b'01110111' ;Met RB[6,5,4,2,1,0] en entre, RB3 en sortie
         TRIS    PORTB

         BSF     PORTB,3     ;Sonde la touche avec RB3 = 1

         BTFSS   PORTB,6     ;Skip si RB6 = 1 (Touche appuye)
         GOTO    KEY06       ;Saut car touche non appuye
         BTFSS   TSTART,7    ;Skip si touche dj appuye
         INCF    TSTART      ;Incrmente compteur
         GOTO    KEY07       ;Saut touche suivante
KEY06    MOVLW   128-4       ;128 == (Bit7 = 1)
         MOVWF   TSTART      ;Le Bit7 passera  1 aprs 4 appels OK

KEY07                        ; Lecture touche ON/OFF (RB4)
         
         MOVLW   b'01101111' ;Met RB[6,5,3,2,1,0] en entre, RB4 en sortie
         TRIS    PORTB

         BSF     PORTB,4     ;Sonde la touche avec RB4 = 1

         BTFSS   PORTB,6     ;Skip si RB6 = 1 (Touche appuye)
         GOTO    KEY08       ;Saut car touche non appuye
         BTFSS   TSTOP,7     ;Skip si touche dj appuye
         INCF    TSTOP       ;Incrmente compteur
         GOTO    KEY09       ;Saut touche suivante
KEY08    MOVLW   128-4       ;128 == (Bit7 = 1)
         MOVWF   TSTOP       ;Le Bit7 passera  1 aprs 4 appels OK

KEY09                        ; Fin
         
         MOVLW   b'00000000' ;Remet RB6 en sortie
         TRIS    PORTB

         RETLW   00          ;Fin procdure lecture du clavier


;
;        Dcrmente [MD MU : SD SU]
;

DECAFF                       ; Units secondes (SU)

         MOVLW   1           ;Dcrmente les units des secondes
         SUBWF   SU,1        ;Destination dans SU
         BTFSC   STAT,CARRY  ;Skip si dpassement (passage de 0  FF, C = 1)
         GOTO    DECFIN
         MOVLW   9           ;Initialise SU  9
         MOVWF   SU

                             ; Dizaines secondes (SD)

         MOVLW   1           ;Dcrmente les dizaines des secondes
         SUBWF   SD,1        ;Destination dans SD
         BTFSC   STAT,CARRY  ;Skip si dpassement (passage de 0  FF, C = 1)
         GOTO    DECFIN
         MOVLW   5           ;Initialise SD  5
         MOVWF   SD

                             ; Units minutes (MU)

         MOVLW   1           ;Dcrmente les units des minutes
         SUBWF   MU,1        ;Destination dans MU
         BTFSC   STAT,CARRY  ;Skip si dpassement (passage de 0  FF, C = 1)
         GOTO    DECFIN
         MOVLW   9           ;Initialise MU  9
         MOVWF   MU

                             ; Dizaines minutes (MD)

         MOVLW   1           ;Dcrmente les dizaines des minutes
         SUBWF   MD,1        ;Destination dans MD
         BTFSC   STAT,CARRY  ;Skip si dpassement (passage de 0  FF, C = 1)
         GOTO    DECFIN
         MOVLW   9           ;Initialise MD  9
         MOVWF   MD

DECFIN   RETLW   00          ;Fin procdure de dcrmentation
         
;
;        Gestion des ordres clavier
;

GESCLAV  BTFSC   ETAT,0      ;Skip si pas de tempo en cours              
         GOTO    G030        ;Ignore touches Minutes et Secondes

; Gestion touche Minute

         BTFSS   TMIN,7      ;Skip si touche Minute On
         GOTO    G00         ;Touche Minute Off

         DECFSZ  REPET0      ;Gestion touche Minute On, teste tempo
         GOTO    G01         ;Touche Minute ignore tant que tempo
         MOVLW   TMPREPET    ;Lance une nouvelle temporisation
         MOVWF   REPET0

         BTFSC   TDOWN,7     ;Skip si touche DOWN Off = Dcrmente
         GOTO    GE0         ;Touche DOWN On = Dcrmente

                             ; Incrmente [MDC MUC] Minutes de consigne

         INCF    MUC         ;Incrmente units minutes ,dest. dans MUC
         MOVLW   10          ;Teste si dpassement
         SUBWF   MUC,W
         BTFSS   STAT,ZBIT   ;Skip si MUC = 10
         GOTO    G01
         CLRF    MUC         ;Initialise MUC  0

         INCF    MDC         ;Incrmente dizaines minutes, dest. dans MDC
         MOVLW   10          ;Teste si dpassement
         SUBWF   MDC,W
         BTFSS   STAT,ZBIT   ;Skip si MDC = 10
         GOTO    G01
         CLRF    MDC         ;Initialise MDC  0

         GOTO    G01

GE0                          ; Dcrmente [MDC MUC] Minutes de consigne

         MOVLW   1           ;Dcrmentation de 1
         SUBWF   MUC         ;Dcrmente units minutes ,dest. dans MUC
         BTFSC   STAT,CARRY  ;Skip si MUC = FF (Dpassement => C = 1)
         GOTO    G01
         MOVLW   9           ;Initialise MUC  9
         MOVWF   MUC

         MOVLW   1           ;Dcrmentation de 1
         SUBWF   MDC         ;Dcrmente dizaines minutes, dest. dans MDC
         BTFSC   STAT,CARRY  ;Skip si MDC = FF (Dpassement => C = 1)
         GOTO    G01
         MOVLW   9           ;Initialise MDC  9
         MOVWF   MDC

         GOTO    G01

G00      MOVLW   1           ;Touche Minute Off
         MOVWF   REPET0      ;Pas de tempo pour un premier appel


; Gestion touche Seconde

G01      BTFSS   TSEC,7      ;Skip si touche Seconde On
         GOTO    G02         ;Touche Seconde Off

         DECFSZ  REPET1      ;Gestion touche Seconde On, teste tempo
         GOTO    G03         ;Touche Seconde ignore tant que tempo
         MOVLW   TMPREPET    ;Lance une nouvelle temporisation
         MOVWF   REPET1

         BTFSC   TDOWN,7     ;Skip si touche DOWN Off = Dcrmente
         GOTO    GE1         ;Touche DOWN On = Dcrmente

                             ; Incrmente [SDC SUC] Minutes de consigne

         INCF    SUC         ;Incrmente units secondes ,dest. dans SUC
         MOVLW   10          ;Teste si dpassement
         SUBWF   SUC,W
         BTFSS   STAT,ZBIT   ;Skip si SUC = 10
         GOTO    G03
         CLRF    SUC         ;Initialise SUC  0

         INCF    SDC         ;Incrmente dizaines secondes, dest. dans SDC
         MOVLW   6           ;Teste si dpassement
         SUBWF   SDC,W
         BTFSS   STAT,ZBIT   ;Skip si SDC = 6
         GOTO    G03
         CLRF    SDC         ;Initialise SDC  0

         GOTO    G03

GE1                          ; Dcrmente [SDC SUC] Minutes de consigne

         MOVLW   1           ;Dcrmentation de 1
         SUBWF   SUC         ;Dcrmente units minutes ,dest. dans SUC
         BTFSC   STAT,CARRY  ;Skip si SUC = FF (Dpassement => C = 1)
         GOTO    G03
         MOVLW   9           ;Initialise SUC  9
         MOVWF   SUC

         MOVLW   1           ;Dcrmentation de 1
         SUBWF   SDC         ;Dcrmente dizaines minutes, dest. dans SDC
         BTFSC   STAT,CARRY  ;Skip si SDC = FF (Dpassement => C = 1)
         GOTO    G03
         MOVLW   5           ;Initialise SDC  5
         MOVWF   SDC

         GOTO    G03
         
G02      MOVLW   1           ;Touche Seconde Off
         MOVWF   REPET1      ;Pas de tempo pour un premier appel

; Gestion touche Stop (ON/OFF)

G03      BTFSS   TSTOP,7     ;Skip si touche Stop appuye
         GOTO    G07
         BTFSC   ETAT,2      ;Skip si touche Stop prcdemment relache
         GOTO    G08
         MOVFW   PORTB       ;Lecture du PORTB
         XORLW   b'10000000' ;Bascule Bit7(PORTB)
         MOVWF   PORTB       ;Ecriture du PORTB
         BSF     ETAT,2      ;Mmorise l'tat 1 de la touche STOP
         GOTO    G08
G07      BCF     ETAT,2      ;Mmorise l'tat 0 de la touche STOP

; Transfert contenu de la consigne  l'affichage

G08      MOVFW   MUC         ;Minute Unit
         MOVWF   MU          
         MOVFW   MDC         ;Minute Dizaines
         MOVWF   MD
         MOVFW   SUC         ;Seconde Units
         MOVWF   SU
         MOVFW   SDC         ;Seconde Dizaines
         MOVWF   SD

; Gestion touche Start (Start/Stop)

G030     BTFSS   TSTART,7    ;Skip si touche Start On
         GOTO    G05
         BTFSC   ETAT,1      ;Skip si touche Start prcdemment Off
         GOTO    G06
         BSF     ETAT,1      ;Mmorise touche Start On
         BTFSC   ETAT,0      ;Skip si pas de tempo en cours
         GOTO    G04

         MOVLW   1           ;Initialise le compteur Timer pour compter ../..
         MOVWF   TIMER       ;une seconde entire au dmarrage de la temporisation
         BSF     ETAT,0      ;Dclare tempo en cours
         MOVFW   PORTB       ;Lecture du PORTB
         XORLW   b'10000000' ;Bascule Bit7(PORTB)
         MOVWF   PORTB       ;Ecriture du PORTB

                             ; Transfert contenu de la consigne  l'affichage

         MOVFW   MUC         ;Minute Unit
         MOVWF   MU          
         MOVFW   MDC         ;Minute Dizaines
         MOVWF   MD
         MOVFW   SUC         ;Seconde Units
         MOVWF   SU
         MOVFW   SDC         ;Seconde Dizaines
         MOVWF   SD
         GOTO    G06

G04      BCF     ETAT,0      ;Dclare pas de tempo en cours
         MOVFW   PORTB       ;Lecture du PORTB
         XORLW   b'10000000' ;Bascule Bit7(PORTB)
         MOVWF   PORTB       ;Ecriture du PORTB
         GOTO    G06

G05      BCF     ETAT,1      ;Mmorise touche Start Off

G06      RETLW   00

;
;        Point d'entre au RESET
;

         ORG     PIC54       ;Reset Vector
         GOTO    MAIN        ;Programme principal

         END


